/*
 * MTV211 Frontend driver
 *
 * Copyright (C) 2008 thara (pu2008@sourceforge.jp)
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License as
 *	published by the Free Software Foundation, version 2.
 *
 */

#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/string.h>
#include <linux/slab.h>

#include "dvb_frontend.h"
#include "mtv211.h"

struct mtv211_state {
	struct i2c_adapter* i2c;
	const struct mtv211_config* config;
	struct dvb_frontend frontend;
};

static int debug;
#define dprintk(args...) \
	do { \
		if (debug) printk(KERN_DEBUG "mtv211: " args); \
	} while (0)

static int mtv211_read_status(struct dvb_frontend* fe, fe_status_t* status)
{
	*status = FE_HAS_SIGNAL
		| FE_HAS_CARRIER
		| FE_HAS_VITERBI
		| FE_HAS_SYNC
		| FE_HAS_LOCK;

	return 0;
}

static int mtv211_read_ber(struct dvb_frontend* fe, u32* ber)
{
	*ber = 0;
	return 0;
}

static int mtv211_read_signal_strength(struct dvb_frontend* fe, u16* strength)
{
	*strength = 0;
	return 0;
}

static int mtv211_read_snr(struct dvb_frontend* fe, u16* snr)
{
	*snr = 0;
	return 0;
}

static int mtv211_read_ucblocks(struct dvb_frontend* fe, u32* ucblocks)
{
	*ucblocks = 0;
	return 0;
}

static int mtv211_get_frontend(struct dvb_frontend* fe, struct dvb_frontend_parameters *p)
{
	return 0;
}

static int mtv211_write(struct mtv211_state* state, const int reg,
		       const void *src, const size_t count)
{
	int ret;
	u8 buf[count + 1];
	struct i2c_msg msg;

	if(debug) {
		int i;
		dprintk("W(%d):", reg & 0x7f);
		for (i = 0; i < count; i++)
			printk(" %02x", ((const u8 *) src)[i]);
		printk("\n");
	}

	buf[0] = reg;
	memcpy(&buf[1], src, count);

	msg.addr = state->config->demod_address;
	msg.flags = 0;
	msg.buf = buf;
	msg.len = count + 1;

	ret = i2c_transfer(state->i2c, &msg, 1);

	if (ret != 1) {
		dprintk("%s: ret == %d\n", __FUNCTION__, ret);
		return -EREMOTEIO;
	}

	return 0;
}


static int mtv211_set_frontend(struct dvb_frontend* fe, struct dvb_frontend_parameters *p)
{
  struct mtv211_state* state = fe->demodulator_priv;
  static char chdata;

  chdata=(p->frequency/1000000 - 395) / 6;
  mtv211_write(state,0x01,&chdata,1);
  
  return 0;
}

static int mtv211_sleep(struct dvb_frontend* fe)
{
	return 0;
}

static int mtv211_init(struct dvb_frontend* fe)
{
	return 0;
}

static void mtv211_release(struct dvb_frontend* fe)
{
	struct mtv211_state* state = fe->demodulator_priv;
	kfree(state);
}


static int mtv211_read(struct mtv211_state* state, const int reg,
		      void *buf, const size_t count)
{
	int ret;
	struct i2c_msg msg[2];
	u8 regbuf[1] = { reg };

	msg[0].addr = state->config->demod_address;
	msg[0].flags = 0;
	msg[0].buf = regbuf;
	msg[0].len = 1;
	msg[1].addr = state->config->demod_address;
	msg[1].flags = I2C_M_RD;
	msg[1].buf = buf;
	msg[1].len = count;

	ret = i2c_transfer(state->i2c, msg, 2);

	if (ret != 2) {
		printk(KERN_ERR "%s: ret == %d\n", __FUNCTION__, ret);
		return -EREMOTEIO;
	}

	if(debug) {
		int i;
		dprintk("R(%d):", reg & 0x7f);
		for (i = 0; i < count; i++)
			printk(" %02x", ((const u8 *) buf)[i]);
		printk("\n");
	}

	return 0;
}

static inline int mtv211_readreg(struct mtv211_state* state,
				const int reg, u8 *val)
{
	return mtv211_read(state, reg, val, 1);
}

static struct dvb_frontend_ops mtv211_ops;

struct dvb_frontend* mtv211_attach(const struct mtv211_config* config,
				   struct i2c_adapter* i2c)
{
  struct mtv211_state* state = NULL;

	/* allocate memory for the internal state */
	state = kmalloc(sizeof(struct mtv211_state), GFP_KERNEL);
	if (state == NULL) goto error;

	/* create dvb_frontend */
	memcpy(&state->frontend.ops, &mtv211_ops, sizeof(struct dvb_frontend_ops));
	state->frontend.demodulator_priv = state;

	state->config = config;
	state->i2c = i2c;

	return &state->frontend;

error:
	kfree(state);
	return NULL;
}

static struct dvb_frontend_ops mtv211_ops = {

	.info = {
		.name			= "MTV211",
		.type			= FE_OFDM,
		.frequency_min		= 0,
		.frequency_max		= 863250000,
		.frequency_stepsize	=   1000000,
		.caps = FE_CAN_FEC_1_2 | FE_CAN_FEC_2_3 | FE_CAN_FEC_3_4 |
				FE_CAN_FEC_4_5 | FE_CAN_FEC_5_6 | FE_CAN_FEC_6_7 |
				FE_CAN_FEC_7_8 | FE_CAN_FEC_8_9 | FE_CAN_FEC_AUTO |
				FE_CAN_QAM_16 | FE_CAN_QAM_64 | FE_CAN_QAM_AUTO |
				FE_CAN_TRANSMISSION_MODE_AUTO |
				FE_CAN_GUARD_INTERVAL_AUTO |
				FE_CAN_HIERARCHY_AUTO,
	},

	.release = mtv211_release,

	.init = mtv211_init,
	.sleep = mtv211_sleep,

	.set_frontend = mtv211_set_frontend,
	.get_frontend = mtv211_get_frontend,

	.read_status = mtv211_read_status,
	.read_ber = mtv211_read_ber,
	.read_signal_strength = mtv211_read_signal_strength,
	.read_snr = mtv211_read_snr,
	.read_ucblocks = mtv211_read_ucblocks,
};

MODULE_DESCRIPTION("MTV211 Frontend");
MODULE_AUTHOR("thara (pu2008@sourceforge.jp)");
MODULE_LICENSE("GPL");

EXPORT_SYMBOL(mtv211_attach);
